from PIL import Image


def mandelbrot(c: complex, m: int) -> int:
    i = 0
    z = 0
    while abs(z) < 2.0 and i < m:
        z = z * z + c
        i += 1
    return i


def sample(z: complex, w: complex,
           x: int, y: int,
           sx: int, sy: int) -> complex:
    x, y = x / sx, y / sy
    return complex((1 - x) * z.real + x * w.real,
                   (1 - y) * z.imag + y * w.imag)


def color(i: int, max_iter: int) -> tuple[int, int, int]:
    hue = int(255 * (i / max_iter))
    value = 255 if i < max_iter else 0
    saturation = 255
    return (hue, saturation, value)


def render_mandelbrot(c_min: complex, c_max: complex,
                      size_x: int, size_y: int,
                      max_iter: int, path: str):
    img = Image.new('HSV', (size_x, size_y))
    for x in range(size_x):
        for y in range(size_y):
            c = sample(c_min, c_max, x, y, size_x, size_y)
            i = mandelbrot(c, max_iter)
            img.putpixel((x, y), color(i, max_iter))
    img.convert('RGB').save(path, quality=95)


if __name__ == '__main__':
    render_mandelbrot(-2-1j, 1+1j, 900, 600, 50, "output.jpg")
